/* Lists.c
   Verwaltung doppelt verlinkter Listen

   (C) Dario Abatianni (eisfuchs@tigress.com) */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Lists.h"
#include "Constants.h"
#include "ErrorLog.h"

// #define DEBUG

#ifdef DEBUG
struct nick
{
  char *nick;
  char *hostmask;
  char op;
  char voice;
  char selected;
};
#endif

struct list *newlist()
{
  struct list *list;

#ifdef DEBUG
  printf("Entering newlist\n");
#endif
  list=calloc(1,sizeof(struct list));

  if(list==0) logerror(errorlogwindow,"newlist(): list==0!");

#ifdef DEBUG
  dumplist(list);
#endif

  return list;
}

struct node *allocnode(unsigned long datalength)
{
  struct node *newnode;
  void *newdata;

#ifdef DEBUG
  printf("Entering allocnode\n");
#endif

  newnode=calloc(1,sizeof(struct node));
  if(newnode==0) logerror(errorlogwindow,"allocnode(): newnode==0!");
  else
  {
    newdata=calloc(1,datalength);
    if(newdata==0)
    {
      logerror(errorlogwindow,"allocnode(): newdata==0!");
      free(newnode);
      newnode=0;
    }
    else
    {
      newnode->data=newdata;
    }
  }

  return newnode;
}

struct node *addnewnodetolist(struct list *list,unsigned long datalength)
{
  struct node *newnode;

#ifdef DEBUG
  printf("Entering addnewnodetolist\n");
#endif
  newnode=0;

  if(list==0) logerror(errorlogwindow,"addnewnodetolist(): list==0!");
  else
  {
    newnode=allocnode(datalength);
    if(newnode==0) logerror(errorlogwindow,"addnewnodetolist(): newnode==0!");
    else
    {
      if(list->head==0)    // Dann mu auch list->tail==0 sein
      {
        list->head=newnode;
        list->tail=newnode;
      }
      else
      {
        newnode->prev=list->tail;
        list->tail->next=newnode;
        list->tail=newnode;
      }

      newnode->list=list;

#ifdef DEBUG
//      dumpnode(newnode);
#endif
    }
  }

#ifdef DEBUG
//  dumplist(list);
#endif

  return newnode;
}

struct node *unlinknode(struct node *node)
{
  struct node *returnnode;
#ifdef DEBUG
  struct list *list;

  printf("Entering unlinknode\n");
#endif

  returnnode=0;

  if(node==0) logerror(errorlogwindow,"unlinknode(): node==0!");
  else
  {
#ifdef DEBUG
    list=node->list;
#endif

    returnnode=(node->next) ? node->next : node->prev;

    if(node->next) node->next->prev=node->prev;
    else node->list->tail=node->prev;

    if(node->prev) node->prev->next=node->next;
    else node->list->head=node->next;

    node->list=0;
  }

#ifdef DEBUG
  dumplist(list);
#endif

  return returnnode;
}

struct node *freenode(struct node *node)
{
  struct node *returnnode;
#ifdef DEBUG
  struct list *list;

  printf("Entering freenode\n");
#endif

  returnnode=0;

  if(node==0) logerror(errorlogwindow,"freenode(): node==0!");
  else
  {
#ifdef DEBUG
    list=node->list;
#endif
    if(node->list) returnnode=unlinknode(node);

    free(node->data);
    free(node);
  }

#ifdef DEBUG
  dumplist(list);
#endif

  return returnnode;
}

struct node *getnodebynumber(struct list *list,unsigned long count)
{
  struct node *returnnode;
  unsigned char found;

  returnnode=0;

#ifdef DEBUG
  printf("Entering getnodebynumber\n");
#endif

  if(list==0) logerror(errorlogwindow,"getnodebynumber(): list==0!");
  {
    found=0;
    returnnode=list->head;

    while(!found && returnnode)
    {
      if(count==0) found=1;
      else
      {
        returnnode=returnnode->next;
        count--;
      }
    } // endwhile
  }

  if(returnnode==0) logerror(errorlogwindow,"getnodebynumber(): returnnode==0!\n");

  return returnnode;
}

struct node *getnodebydata(struct list *list,void *data,unsigned long len)
{
  struct node *returnnode;
  unsigned char found;

  returnnode=0;
#ifdef DEBUG
  printf("Entering getnodebydata\n");
#endif

  if(list==0) logerror(errorlogwindow,"getnodebydata(): list==0!");
  else
  {
    if(data==0) logerror(errorlogwindow,"getnodebydata(): data==0!");
    else
    {
      found=0;
      returnnode=list->head;

      while(!found && returnnode)
      {
        if(memcmp(returnnode->data,data,len)==0) found=1;
        else
        {
          returnnode=returnnode->next;
        }
      } // endwhile
    }
  }
  return returnnode;
}

unsigned long listlen(struct list *list)
{
  struct node *looknode;
  unsigned long count;

#ifdef DEBUG
  printf("Entering listlen\n");
#endif

  count=0;

  if(list==0) logerror(errorlogwindow,"listlen(): list==0!");
  else
  {
    looknode=list->head;
    while(looknode)
    {
      looknode=looknode->next;
      count++;
    }
  }

  return count;
}

#ifdef DEBUG
void dumpnode(struct node *node)
{
  struct nick *nick;

  if(node==0) logerror(errorlogwindow,"dumpnode(): node==0!");
  else
  {
    nick=node->data;
    printf("node address: %0.8x\n",node);
    printf("node->list=%0.8x\n",node->list);
    printf("node->prev=%0.8x\n",node->prev);
    printf("node->next=%0.8x\n",node->next);
    printf("node->data->nick ='%s'\n",nick->nick);
    printf("node->data->op   =%d\n",nick->op);
    printf("node->data->voice=%d\n",nick->voice);

    printf("node->data=%0.8x\n\n",node->data);
  }
}

void dumplistheader(struct list *list)
{
  if(list==0) logerror(errorlogwindow,"dumplistheader(): list==0!");
  else
  {
    printf("List address: %08lx\n",(unsigned long) list);

    printf("list->head=%08lx\n",(unsigned long) list->head);
    printf("list->tail=%08lx\n\n",(unsigned long) list->tail);
  }
}

void dumplist(struct list *list)
{
  struct node *node;

  if(list==0) logerror(errorlogwindow,"dumplist(): list==0!");
  else
  {
    dumplistheader(list);
    node=list->head;

    while(node)
    {
      dumpnode(node);
      node=node->next;
    } /* endwhile */
  }
}

#endif

void clearlist(struct list *list)
{
#ifdef DEBUG
  printf("Entering clearlist\n");
#endif
  if(list==0) logerror(errorlogwindow,"clearlist(): list==0!");
  else
  {
    while(list->head)
    {
#ifdef DEBUG
      dumpnode(list->head);
#endif
      freenode(list->head);
    }
  }
#ifdef DEBUG
  dumplist(list);
#endif
}

void freelist(struct list *list)
{
#ifdef DEBUG
  printf("Entering freelist\n");
#endif
  if(list==0) logerror(errorlogwindow,"freelist(): list==0!");
  else
  {
    clearlist(list);

    free(list);
  }
}
