/* Preferences.c
   Ein Teil von FoxIRC (C)by Dario Abatianni (eisfuchs@tigress.com)

   Einlesen, ausgeben und ndern der Voreinstellungen.  */

#include <string.h>
#include <pwd.h>

#include "Lists.h"
#include "Events.h"
#include "Alias.h"
#include "Preferences.h"
#include "Ignore.h"
#include "ErrorLog.h"

extern char *version;
extern struct list *eventlist;
// extern errorlogwindow;

struct servernode *addserverprefs(struct preferences *prefs,char *servername)
{
  struct servernode *node,*looknode;

  node=calloc(sizeof(struct servernode),1);

  if(node==0) logerror(errorlogwindow,"addserverprefs(): node==0!");
  else
  {
    if(prefs->serverlist==0) prefs->serverlist=node;
    else
    {
      looknode=prefs->serverlist;
      while(looknode->next!=0) looknode=looknode->next;
      looknode->next=node;
      node->prev=looknode;
    }
    strcpy(node->name,servername);
    prefs->numofservers++;
  }
  return node;
}

int freadstr(FILE *handle,char *string)
{
  char quit;
  int c,count;

  string[0]=0;
  quit=0;
  count=0;

  while(quit==0)
  {
    c=fgetc(handle);
    if(c==13 || c==10 || c==EOF) quit=1; else string[count++]=c;
  }

  string[count]=0;
  if(c==EOF) count=EOF;
  return count;
}

void getcols(char *string,int *i,int *r,int *g,int *b)
{
  char helpstr[4];
  int count,outcount,index,cols[4];

  count=0;
  for(index=0;index<4;index++)
  {
    outcount=0;
    while(string[count]!=',' && string[count]!=0) helpstr[outcount++]=string[count++];
    helpstr[outcount]=0;
    cols[index]=atoi(helpstr);
    count++;
  }

  *i=cols[0];
  *r=cols[1];
  *g=cols[2];
  *b=cols[3];
}

void writeprefs(struct preferences *prefs)
{
  FILE *handle;
  int count;
  struct servernode *node;
  struct node *lookignorenode;
  struct ignore *lookignore;
  struct node *lookeventnode;
  struct event *lookevent;

  if(prefs->serverlist==0)
  {
    node=addserverprefs(prefs,"localhost");
    node->port=6667;
  }
  if(prefs->chosenservernum==0) prefs->chosenservernum=1;

  if((handle=fopen(prefs->prefspath,"w"))!=0)
  {
    fprintf(handle,"# FoxIRC-Konfigurationsdatei\n"
                   "# Achtung! Zeilen, die nicht zur Konfiguration gehoeren, werden bei\n"
                   "# der naechsten Sicherung automatisch entfernt!\n"
                   "# Es wird dringend davon abgeraten, die Datei per Hand zu aendern!\n\n");

    fprintf(handle,"# Programmversion (steuert das Update-Fenster)\n\n");
    fprintf(handle,"VERSION:%s\n\n",version);

    fprintf(handle,"# Serverliste\n\n");

    node=prefs->serverlist;
    while(node!=0)
    {
      fprintf(handle,"SERVER:%s\n"    ,node->name);
      fprintf(handle,"CHANNEL:%s\n" ,node->channel);
      if(node->password) fprintf(handle,"KEYWORD:%s\n" ,node->password);
      fprintf(handle,"PORT:%d\n\n"      ,node->port);
      node=node->next;
    }

    fprintf(handle,"# Momentan aktiver Server\n\n");
    fprintf(handle,"CHOSEN:%d\n\n"  ,prefs->chosenservernum);

    fprintf(handle,"# DCC-Pfad\n\n");
    if(prefs->dccpath)
    {
      fprintf(handle,"DCCPATH:%s",prefs->dccpath);
      if(prefs->dccpath[strlen(prefs->dccpath)-1]!='/') fprintf(handle,"/");
    }
    else
    {
      struct passwd *user;
      user=getpwuid(geteuid());

      if(user==0)
      {
        logerror(errorlogwindow,"writeprefs(): Could not get user identity!");
      }
      else
      {
        prefs->dccpath=malloc(strlen(user->pw_dir)+1);
        strcpy(prefs->dccpath,user->pw_dir);
      }
    }
    fprintf(handle,"\n\n");

    fprintf(handle,"# Nickname-Liste\n\n");
    fprintf(handle,"NICK1:%s\n"     ,prefs->nick[0]);
    fprintf(handle,"NICK2:%s\n"     ,prefs->nick[1]);
    fprintf(handle,"NICK3:%s\n"     ,prefs->nick[2]);
    fprintf(handle,"NICK4:%s\n\n"   ,prefs->nick[3]);

    fprintf(handle,"# Identitaet\n\n");
    fprintf(handle,"REALNAME:%s\n"  ,prefs->realname);
    fprintf(handle,"IDENT:%s\n\n"   ,prefs->ident);

    fprintf(handle,"# PART, QUIT und KICK-Meldungen\n\n");
    fprintf(handle,"QUITMSG:%s\n"   ,prefs->quitmsg);
    fprintf(handle,"KICKMSG:%s\n\n" ,prefs->kickmsg);

    fprintf(handle,"# Browser-Kommandozeile\n\n");
    fprintf(handle,"BROWSER:%s\n\n"   ,prefs->browsercommand);

    fprintf(handle,"# Nickname-Ueberwachung\n\n");
    fprintf(handle,"NOTIFY:%s\n"    ,prefs->notifystring);
    fprintf(handle,"DELAY:%d\n\n"   ,prefs->notifydelay);

    fprintf(handle,"# Hervorhebungs-Muster\n\n");
    fprintf(handle,"HILIGHT:%s\n\n" ,prefs->hilightstring);

    fprintf(handle,"# Ignorierte Nicks\n");
    if(prefs->ignorelist)
    {
      lookignorenode=prefs->ignorelist->head;
      if(lookignore) fprintf(handle,"\n");
      while(lookignorenode)
      {
        lookignore=lookignorenode->data;
        fprintf(handle,"IGNORE:%s%s\n",(lookignore->query==0) ? "-" : "+",lookignore->name);
        lookignorenode=lookignorenode->next;
      } // endwhile
    }

    fprintf(handle,"\n# Hoehe der Zeichen im Kanal und der Nickliste\n\n");
    fprintf(handle,"FONTCHAN:%d\n"  ,prefs->channelsize);
    fprintf(handle,"FONTNICK:%d\n\n",prefs->nicksize);

    fprintf(handle,"# Flags\n\n");
    fprintf(handle,"%s\n",  (prefs->motd==1)            ? "MOTD"        : "NOMOTD");
    fprintf(handle,"%s\n",  (prefs->fixedmotd==1)       ? "FIXEDMOTD"   : "NOFIXEDMOTD");
    fprintf(handle,"%s\n",  (prefs->log==1)             ? "LOG"         : "NOLOG");
    fprintf(handle,"%s\n",  (prefs->logtimestamp==1)    ? "TIMESTAMP"   : "NOTIMESTAMP");
    fprintf(handle,"%s\n",  (prefs->smalllog==1)        ? "SMALLLOG"    : "NOSMALLLOG");
    fprintf(handle,"%s\n",  (prefs->openquery==1)       ? "QUERY"       : "NOQUERY");
    fprintf(handle,"%s\n",  (prefs->sortnicks==1)       ? "SORTNICKS"   : "NOSORTNICKS");
    fprintf(handle,"%s\n",  (prefs->notify==1)          ? "NICKNOTIFY"  : "NONICKNOTIFY");
    fprintf(handle,"%s\n",  (prefs->autonotify==1)      ? "AUTONOTIFY"  : "NOAUTONOTIFY");
    fprintf(handle,"%s\n",  (prefs->events==1)          ? "EVENTS"      : "NOEVENTS");
    fprintf(handle,"%s\n",  (prefs->hilight==1)         ? "HILIGHT"     : "NOHILIGHT");
    fprintf(handle,"%s\n",  (prefs->querybell==1)       ? "QUERYBELL"   : "NOQUERYBELL");
    fprintf(handle,"%s\n",  (prefs->queryalwaysbell==1) ? "QUERYALWAYS" : "NOQUERYALWAYS");
    fprintf(handle,"%s\n",  (prefs->channelbell==1)     ? "CHANNELBELL" : "NOCHANNELBELL");
    fprintf(handle,"%s\n",  (prefs->sketchpad==1)       ? "SKETCHPAD"   : "NOSKETCHPAD");
    fprintf(handle,"%s\n",  (prefs->serverpings==1)     ? "SERVERPINGS" : "NOSERVERPINGS");
    fprintf(handle,"%s\n",  (prefs->decodemorse==1)     ? "DECODEMORSE" : "NODECODEMORSE");
    fprintf(handle,"%s\n\n",(prefs->joinoninvite==1)    ? "JOINONINVITE": "NOJOINONINVITE");

    fprintf(handle,"# Auto-Away\n\n");
    fprintf(handle,"%s\n",  (prefs->autoaway==1)        ? "AUTOAWAY"    : "NOAUTOAWAY");
    fprintf(handle,"AUTOAWAYTIME:%d\n",prefs->autoawaytime);
    fprintf(handle,"AUTOAWAYTEXT:%s\n\n",(prefs->autoawaytext) ? prefs->autoawaytext : "");

    fprintf(handle,"# Farbeinstellungen\n\n");
    for(count=0;count<9;count++) fprintf(handle,"COL:%d,%d,%d,%d\n",count,prefs->color[count][0],prefs->color[count][1],prefs->color[count][2]);
    fprintf(handle,"\n# Knopffunktionen\n\n");
    for(count=0;count<8;count++) fprintf(handle,"BUTTON:%d,%s\n",count,prefs->button[count]);
    fprintf(handle,"\n# Knopfnamen\n\n");
    for(count=0;count<8;count++) fprintf(handle,"BUTTONNAME:%d,%s\n",count,prefs->butnam[count]);

    fprintf(handle,"\n# Doppelklickaktion in der Nickliste\n\n");
    fprintf(handle,"NICKLISTACTION:%s\n",prefs->nicklistaction);

    fprintf(handle,"\n# Aktionen\n\n");

    if(prefs->numofactions!=0)
    {
      fprintf(handle,"NUMOFACTIONS:%d\n",prefs->numofactions);

      for(count=0;count<prefs->numofactions;count++)
      {
        fprintf(handle,"\nACTIONTITLE:%s\n",prefs->actiontitle[count]);
        fprintf(handle,"ACTION:%s\n"       ,prefs->actionlist [count]);
      }
    }

    if(listlen(prefs->aliaslist)!=0)
    {
      struct node *aliasnode;

      fprintf(handle,"\n# Aliases\n\n");

      aliasnode=prefs->aliaslist->head;

      do
      {
        struct alias *alias;

        alias=aliasnode->data;

        fprintf(handle,"ALIAS:%s,%s\n",alias->title,alias->action);

        aliasnode=aliasnode->next;
      } while(aliasnode);
    }

    if(eventlist)
    {
      lookeventnode=eventlist->head;
      fprintf(handle,"\n# Ereignisse\n");

      while(lookeventnode)
      {
        lookevent=lookeventnode->data;
        fprintf(handle,"\nEVENTNAME:%s\n",lookevent->name);
        fprintf(handle,"EVENTTYPE:%d\n",lookevent->type);
        fprintf(handle,"EVENTPARAMETER:%s\n",lookevent->parameter);
        fprintf(handle,"EVENTTEXT:%s\n",lookevent->text);
        fprintf(handle,"EVENTSCRIPT:%s\n",lookevent->script);
        fprintf(handle,"EVENTSOUND:%s\n",lookevent->sound);

        lookeventnode=lookeventnode->next;
      }
    }

    fprintf(handle,"\n# Fenstergren und -positionen\n\n");
    fprintf(handle,"GEOMAIN:%d,%d,%d,%d\n",prefs->geomain.x,prefs->geomain.y,prefs->geomain.w,prefs->geomain.h);
    fprintf(handle,"GEOISON:%d,%d,%d,%d\n",prefs->geoison.x,prefs->geoison.y,prefs->geoison.w,prefs->geoison.h);

    fprintf(handle,"\n# Ende der FoxIRC-Konfigurationsdatei\n");
    fclose(handle);
  }
  else printf("Konnte Preferences-Datei \"%s\" nicht speichern.\n",prefs->prefspath);
}

void allocactions(struct preferences *prefs,int numof)
{
  prefs->numofactions=numof;

  prefs->actionlist =calloc(numof*sizeof(char *),1);
  if(prefs->actionlist==0) printf("allocactions(): prefs->actionlist==0!\n");
  else
  {
    prefs->actiontitle=calloc(numof*sizeof(char *),1);
    if(prefs->actiontitle==0) printf("allocactions(): prefs->actiontitle==0!\n");
  }
}

void readprefs(struct preferences *prefs)
{
  char line[600],*event[5];
  int i,r,g,b,count,actioncount,type;
  struct servernode *node;
  FILE *handle;

  freeprefs(prefs);
  actioncount=0;

  for(count=0;count<5;count++) event[count]=0;

  if((handle=fopen(prefs->prefspath,"r"))!=0)
  {
    while((freadstr(handle,line))!=EOF)
    {
      if(strncasecmp(line,"VERSION:" ,8)==0)
      {
        if(strcmp(version,line+8)==0) prefs->currentversion=1;
      }

      if(strncasecmp(line,"SERVER:"  ,7)==0) node=addserverprefs(prefs,line+7);
      if(strncasecmp(line,"PORT:"    ,5)==0) node->port=atoi(line+5);
      if(strncasecmp(line,"CHOSEN:",  7)==0) prefs->chosenservernum=atoi(line+7);
      if(strncasecmp(line,"CHANNEL:" ,8)==0) strcpy(node->channel,line+8);
      if(strncasecmp(line,"KEYWORD:" ,8)==0)
      {
        int len;

        len=strlen(line+8);
        if(len)
        {
          node->password=malloc(len+1);
          if(node->password)
          {
            strcpy(node->password,line+8);
          }
        }
      }

      if(strncasecmp(line,"DCCPATH:" ,8)==0)
      {
        char slash;

        slash=(line[strlen(line)-1]=='/') ? 0 : 1;

        count=strlen(line+8);
        prefs->dccpath=malloc(count+slash);
        if(prefs->dccpath==0) logerror(errorlogwindow,"readprefs(): Kein Speicher fr DCC-Pfad!");
        else
        {
          strcpy(prefs->dccpath,line+8);
          if(slash) strcat(prefs->dccpath,"/");
        }
      }

      if(strncasecmp(line,"NICK:"    ,5)==0) strcpy(prefs->nick[0],line+5);  // Kompatibilitt zu <0.16
      if(strncasecmp(line,"NICK1:"   ,6)==0) strcpy(prefs->nick[0],line+6);
      if(strncasecmp(line,"NICK2:"   ,6)==0) strcpy(prefs->nick[1],line+6);
      if(strncasecmp(line,"NICK3:"   ,6)==0) strcpy(prefs->nick[2],line+6);
      if(strncasecmp(line,"NICK4:"   ,6)==0) strcpy(prefs->nick[3],line+6);
      if(strncasecmp(line,"REALNAME:",9)==0) strcpy(prefs->realname,line+9);
      if(strncasecmp(line,"IDENT:"   ,6)==0) strcpy(prefs->ident,line+6);
      if(strncasecmp(line,"QUITMSG:" ,8)==0) strcpy(prefs->quitmsg,line+8);
      if(strncasecmp(line,"KICKMSG:" ,8)==0) strcpy(prefs->kickmsg,line+8);
      if(strncasecmp(line,"BROWSER:" ,8)==0)
      {
        if(prefs->browsercommand) free(prefs->browsercommand);
        prefs->browsercommand=calloc(1,strlen(line+8)+1);
        if(prefs->browsercommand) strcpy(prefs->browsercommand,line+8);
      }

      if(strncasecmp(line,"NOTIFY:"  ,7)==0) strcpy(prefs->notifystring,line+7);
      if(strncasecmp(line,"DELAY:"   ,6)==0) prefs->notifydelay=atoi(line+6);

      if(strncasecmp(line,"HILIGHT:" ,8)==0) strcpy(prefs->hilightstring,line+8);

      if(strncasecmp(line,"IGNORE:"  ,7)==0) addignore(prefs->ignorelist,line+8,(line[7]=='+') ? 1 : 0);

      if(strncasecmp(line,"FONTCHAN:",9)==0) prefs->channelsize=atoi(line+9);
      if(strncasecmp(line,"FONTNICK:",9)==0) prefs->nicksize=atoi(line+9);

      if(strncasecmp(line,"COL:"     ,4)==0)
      {
        getcols(line+4,&i,&r,&g,&b);

        if(i>=0 && i<=(COL_ACTION-FL_FREE_COL1))
        {
          prefs->color[i][0]=r;
          prefs->color[i][1]=g;
          prefs->color[i][2]=b;
        }
        else printf("readprefs(): Color Index %d out of range! Ignoring.\n",i);
      }

      if(strncasecmp(line,"BUTTON:"  ,7)==0)
      {
        count=line[7]-0x30;
        strcpy(prefs->button[count],line+9);
      }

      if(strncasecmp(line,"BUTTONNAME:"  ,11)==0)
      {
        count=line[11]-0x30;
        strcpy(prefs->butnam[count],line+13);
      }

      if(strncasecmp(line,"NICKLISTACTION:",15)==0)
      {
        strcpy(prefs->nicklistaction,line+15);
      }

      if(strncasecmp(line,"NUMOFACTIONS:",13)==0)
      {
        allocactions(prefs,atoi(line+13));
      }

      if(strncasecmp(line,"ACTIONTITLE:" ,12)==0)
      {
        prefs->actiontitle[actioncount]=malloc(strlen(line+12)+1);
        strcpy(prefs->actiontitle[actioncount],line+12);
      }

      if(strncasecmp(line,"ACTION:"       ,7)==0)
      {
        prefs->actionlist[actioncount]=malloc(strlen(line+7)+1);
        strcpy(prefs->actionlist[actioncount++],line+7);
      }

      if(strncasecmp(line,"ALIAS:"        ,6)==0)
      {
        if(prefs->aliaslist==0) prefs->aliaslist=newlist();

        if(prefs->aliaslist==0) logerror(errorlogwindow,"readprefs(): prefs->aliaslist==0!");
        else
        {
          char *comma;

          comma=strstr(line,",");
          if(comma==0) logerror(errorlogwindow,"readprefs(): Alias-Definition ohne ','!");
          else
          {
            line[comma-line]=0;
            addalias(prefs->aliaslist,line+6,comma+1);
          }
        }
      }

      if(strncasecmp(line,"EVENTNAME:"   ,10)==0)
      {
        if(event[0]) free(event[0]);
        event[0]=malloc(strlen(line+10)+1);
        strcpy(event[0],line+10);
      }

      if(strncasecmp(line,"EVENTTYPE:"   ,10)==0)
      {
        type=atoi(line+10);
      }

      if(strncasecmp(line,"EVENTPARAMETER:",15)==0)
      {
        if(event[1]) free(event[1]);
        event[1]=malloc(strlen(line+15)+1);
        strcpy(event[1],line+15);
      }

      if(strncasecmp(line,"EVENTTEXT:"   ,10)==0)
      {
        if(event[2]) free(event[2]);
        event[2]=malloc(strlen(line+10)+1);
        strcpy(event[2],line+10);
      }

      if(strncasecmp(line,"EVENTSCRIPT:" ,12)==0)
      {
        if(event[3]) free(event[3]);
        event[3]=malloc(strlen(line+12)+1);
        strcpy(event[3],line+12);
      }

      if(strncasecmp(line,"EVENTSOUND:"  ,11)==0)
      {
        if(event[4]) free(event[4]);
        event[4]=malloc(strlen(line+11)+1);
        strcpy(event[4],line+11);
        addevent(type,event[0],event[1],event[2],event[3],event[4]);
      }

      if(strncasecmp(line,"GEOMAIN:"      ,8)==0)
      {
        sscanf(line,"GEOMAIN:%d,%d,%d,%d\n",&prefs->geomain.x,&prefs->geomain.y,&prefs->geomain.w,&prefs->geomain.h);
//        printf("GEOMAIN:%d,%d,%d,%d\n",prefs->geomain.x,prefs->geomain.y,prefs->geomain.w,prefs->geomain.h);
      }

      if(strncasecmp(line,"GEOISON:"      ,8)==0)
      {
        sscanf(line,"GEOISON:%d,%d,%d,%d\n",&prefs->geoison.x,&prefs->geoison.y,&prefs->geoison.w,&prefs->geoison.h);
//        printf("GEOISON:%d,%d,%d,%d\n",prefs->geoison.x,prefs->geoison.y,prefs->geoison.w,prefs->geoison.h);
      }

      if(strcasecmp(line,"MOTD")==0) prefs->motd=1;
      if(strcasecmp(line,"NOMOTD")==0) prefs->motd=0;
      if(strcasecmp(line,"FIXEDMOTD")==0) prefs->fixedmotd=1;
      if(strcasecmp(line,"NOFIXEDMOTD")==0) prefs->fixedmotd=0;
      if(strcasecmp(line,"LOG")==0) prefs->log=1;
      if(strcasecmp(line,"NOLOG")==0) prefs->log=0;
      if(strcasecmp(line,"TIMESTAMP")==0) prefs->logtimestamp=1;
      if(strcasecmp(line,"NOTIMESTAMP")==0) prefs->logtimestamp=0;
      if(strcasecmp(line,"SMALLLOG")==0) prefs->smalllog=1;
      if(strcasecmp(line,"NOSMALLLOG")==0) prefs->smalllog=0;
      if(strcasecmp(line,"QUERY")==0) prefs->openquery=1;
      if(strcasecmp(line,"NOQUERY")==0) prefs->openquery=0;
      if(strcasecmp(line,"SORTNICKS")==0) prefs->sortnicks=1;
      if(strcasecmp(line,"NOSORTNICKS")==0) prefs->sortnicks=0;
      if(strcasecmp(line,"NICKNOTIFY")==0) prefs->notify=1;
      if(strcasecmp(line,"NONICKNOTIFY")==0) prefs->notify=0;
      if(strcasecmp(line,"AUTONOTIFY")==0) prefs->autonotify=1;
      if(strcasecmp(line,"NOAUTONOTIFY")==0) prefs->autonotify=0;
      if(strcasecmp(line,"EVENTS")==0) prefs->events=1;
      if(strcasecmp(line,"NOEVENTS")==0) prefs->events=0;
      if(strcasecmp(line,"HILIGHT")==0) prefs->hilight=1;
      if(strcasecmp(line,"NOHILIGHT")==0) prefs->hilight=0;
      if(strcasecmp(line,"QUERYBELL")==0) prefs->querybell=1;
      if(strcasecmp(line,"NOQUERYBELL")==0) prefs->querybell=0;
      if(strcasecmp(line,"QUERYALWAYS")==0) prefs->queryalwaysbell=1;
      if(strcasecmp(line,"NOQUERYALWAYS")==0) prefs->queryalwaysbell=0;
      if(strcasecmp(line,"CHANNELBELL")==0) prefs->channelbell=1;
      if(strcasecmp(line,"NOCHANNELBELL")==0) prefs->channelbell=0;
      if(strcasecmp(line,"SKETCHPAD")==0) prefs->sketchpad=1;
      if(strcasecmp(line,"NOSKETCHPAD")==0) prefs->sketchpad=0;
      if(strcasecmp(line,"SERVERPINGS")==0) prefs->serverpings=1;
      if(strcasecmp(line,"NOSERVERPINGS")==0) prefs->serverpings=0;
      if(strcasecmp(line,"DECODEMORSE")==0) prefs->decodemorse=1;
      if(strcasecmp(line,"NODECODEMORSE")==0) prefs->decodemorse=0;
      if(strcasecmp(line,"JOINONINVITE")==0) prefs->joinoninvite=1;
      if(strcasecmp(line,"NOJOINONINVITE")==0) prefs->joinoninvite=0;
      if(strcasecmp(line,"AUTOAWAY")==0) prefs->autoaway=1;
      if(strcasecmp(line,"NOAUTOAWAY")==0) prefs->autoaway=0;

      if(strncasecmp(line,"AUTOAWAYTIME:" ,13)==0)
      {
        prefs->autoawaytime=atoi(line+13);
      }

      if(strncasecmp(line,"AUTOAWAYTEXT:" ,13)==0)
      {
        prefs->autoawaytext=malloc(strlen(line+13)+1);
        if(prefs->autoawaytext==0) printf("readprefs(): prefs->autoawaytext==0!\n");
        else strcpy(prefs->autoawaytext,line+13);
      }
    }

    fclose(handle);

    if(prefs->serverlist==0)
    {
      node=addserverprefs(prefs,"localhost");
      node->port=6667;
    }
    if(prefs->chosenservernum==0) prefs->chosenservernum=1;
    if(prefs->chosenservernum>prefs->numofservers) prefs->chosenservernum=prefs->numofservers;
  }
  else
  {
    writeprefs(prefs);
  }

  for(count=0;count<5;count++) if(event[count]) free(event[count]);
}

void removeserverprefs(struct preferences *prefs,struct servernode *node)
{
  if(node->password) free(node->password);
  if(node->prev!=0) node->prev->next=node->next; else prefs->serverlist=node->next;
  if(node->next!=0) node->next->prev=node->prev;
  free(node);
}

void removeserverprefsnum(struct preferences *prefs,int num)
{
  struct servernode *node;

  node=prefs->serverlist;

  while(--num!=0) node=node->next;
  removeserverprefs(prefs,node);
}

struct servernode *getservernodebynum(struct preferences *prefs,int num)
{
  struct servernode *node;

  node=prefs->serverlist;

  while(--num!=0) node=node->next;

  return node;
}

void freeactions(struct preferences *prefs)
{
  int count;

  if(prefs->numofactions!=0)
  {
    for(count=0;count<prefs->numofactions;count++)
    {
      free(prefs->actionlist[count]);
      free(prefs->actiontitle[count]);
    }

    free(prefs->actionlist);
    free(prefs->actiontitle);

    prefs->numofactions=0;
  }
}

void freeignore(struct preferences *prefs)
{
  struct node *lookignorenode,*nextignorenodde;  /* nextignorenode unsinnig? */
  struct ignore *lookignore;

  if(prefs->ignorelist)
  {
    lookignorenode=prefs->ignorelist->head;
    while(lookignorenode)
    {
      lookignore=lookignorenode->data;

      if(lookignore->name) free(lookignore->name);
      lookignorenode=freenode(lookignorenode);
    } // endwhile
  }
}

void freeprefs(struct preferences *prefs)
{
  struct servernode *node,*nextnode;

  node=prefs->serverlist;

  while(node!=0)
  {
    nextnode=node->next;
    free(node);
    node=nextnode;
  }

  prefs->serverlist=0;
  prefs->numofservers=0;

  freeactions(prefs);
  freeignore(prefs);
  freealiases(prefs);

  freeevents();

  free(prefs->dccpath);

  prefs->dccpath=0;
  prefs->currentversion=0;
}
